@@ -1,34 +1,5 @@
-/*
- * Copyright 2019-2020, Intel Corporation
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above copyright
- *       notice, this list of conditions and the following disclaimer in
- *       the documentation and/or other materials provided with the
- *       distribution.
- *
- *     * Neither the name of the copyright holder nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+// SPDX-License-Identifier: BSD-3-Clause
+/* Copyright 2019-2020, Intel Corporation */
 
 /**
  * @file
@@ -2247,7 +2218,7 @@ public:
 	 * failed.
 	 * @throw pmem::transaction_free_error when freeing old underlying array
 	 * failed.
-	 * @throw rethrows constructor exception.
+	 * @throw rethrows constructor's exception.
 	 */
 	concurrent_hash_map &
 	operator=(const concurrent_hash_map &table)
@@ -2269,7 +2240,7 @@ public:
 	 * failed.
 	 * @throw pmem::transaction_free_error when freeing old underlying array
 	 * failed.
-	 * @throw rethrows constructor exception.
+	 * @throw rethrows constructor's exception.
 	 */
 	concurrent_hash_map &
 	operator=(std::initializer_list<value_type> il)
@@ -2301,7 +2272,7 @@ public:
 	 */
 	void clear();
 
-	/*
+	/**
 	 * Should be called before concurrent_hash_map destructor is called.
 	 * Otherwise, program can terminate if an exception occurs while freeing
 	 * memory inside dtor.
@@ -2316,6 +2287,9 @@ public:
 	void
 	free_data()
 	{
+		if (!this->tls_ptr)
+			return;
+
 		auto pop = get_pool_base();
 
 		transaction::run(pop, [&] {
@@ -2325,11 +2299,20 @@ public:
 	}
 
 	/**
-	 * Clear table and destroy it.
+	 * free_data should be called before concurrent_hash_map
+	 * destructor is called. Otherwise, program can terminate if
+	 * an exception occurs while freeing memory inside dtor.
+	 *
+	 * Hash map can NOT be used after free_data() was called (unless it
+	 * was called in a transaction and that transaction aborted).
 	 */
 	~concurrent_hash_map()
 	{
-		free_data();
+		try {
+			free_data();
+		} catch (...) {
+			std::terminate();
+		}
 	}
 
 	//------------------------------------------------------------------------
