@@ -1,34 +1,5 @@
-/*
- * Copyright 2016-2019, Intel Corporation
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above copyright
- *       notice, this list of conditions and the following disclaimer in
- *       the documentation and/or other materials provided with the
- *       distribution.
- *
- *     * Neither the name of the copyright holder nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+// SPDX-License-Identifier: BSD-3-Clause
+/* Copyright 2016-2019, Intel Corporation */
 
 /**
  * @file
@@ -38,11 +9,13 @@
 #ifndef LIBPMEMOBJ_CPP_TRANSACTION_HPP
 #define LIBPMEMOBJ_CPP_TRANSACTION_HPP
 
+#include <array>
 #include <functional>
 #include <string>
+#include <vector>
 
 #include <libpmemobj++/detail/common.hpp>
-#include <libpmemobj++/detail/pexceptions.hpp>
+#include <libpmemobj++/pexceptions.hpp>
 #include <libpmemobj++/pool.hpp>
 #include <libpmemobj/tx_base.h>
 
@@ -66,6 +39,12 @@ namespace obj
  * This class also exposes a closure-like transaction API, which is the
  * preferred way of handling transactions.
  *
+ * This API should NOT be mixed with C transactions API. One issue is that
+ * C++ callbacks registered using transaction::register_callback() would not
+ * be called if C++ transaction is created inside C transaction.
+ * The same is true if user calls pmemobj_tx_set_user_data() inside a C++
+ * transaction.
+ *
  * The typical usage example would be:
  * @snippet doc_snippets/transaction.cpp general_tx_example
  */
@@ -107,17 +86,31 @@ public:
 		template <typename... L>
 		manual(obj::pool_base &pop, L &... locks)
 		{
-			if (pmemobj_tx_begin(pop.handle(), nullptr,
-					     TX_PARAM_NONE) != 0)
-				throw transaction_error(
-					"failed to start transaction");
+			int ret = 0;
+
+			if (pmemobj_tx_stage() == TX_STAGE_NONE) {
+				ret = pmemobj_tx_begin(pop.handle(), nullptr,
+						       TX_PARAM_CB,
+						       transaction::c_callback,
+						       nullptr, TX_PARAM_NONE);
+			} else {
+				ret = pmemobj_tx_begin(pop.handle(), nullptr,
+						       TX_PARAM_NONE);
+			}
+
+			if (ret != 0)
+				throw pmem::transaction_error(
+					"failed to start transaction")
+					.with_pmemobj_errormsg();
 
 			auto err = add_lock(locks...);
 
 			if (err) {
 				pmemobj_tx_abort(EINVAL);
 				(void)pmemobj_tx_end();
-				throw transaction_error("failed to add lock");
+				throw pmem::transaction_error(
+					"failed to add lock")
+					.with_pmemobj_errormsg();
 			}
 		}
 
@@ -230,7 +223,8 @@ public:
 			else if (pmemobj_tx_stage() == TX_STAGE_ONABORT ||
 				 (pmemobj_tx_stage() == TX_STAGE_FINALLY &&
 				  pmemobj_tx_errno() != 0))
-				throw transaction_error("Transaction aborted");
+				throw pmem::transaction_error(
+					"Transaction aborted");
 		}
 
 		/**
@@ -325,10 +319,11 @@ public:
 	abort(int err)
 	{
 		if (pmemobj_tx_stage() != TX_STAGE_WORK)
-			throw transaction_error("wrong stage for abort");
+			throw pmem::transaction_error("wrong stage for abort");
 
 		pmemobj_tx_abort(err);
-		throw manual_tx_abort("explicit abort " + std::to_string(err));
+		throw pmem::manual_tx_abort("explicit abort " +
+					    std::to_string(err));
 	}
 
 	/**
@@ -345,7 +340,7 @@ public:
 	commit()
 	{
 		if (pmemobj_tx_stage() != TX_STAGE_WORK)
-			throw transaction_error("wrong stage for commit");
+			throw pmem::transaction_error("wrong stage for commit");
 
 		pmemobj_tx_commit();
 	}
@@ -397,17 +392,31 @@ public:
 	static void
 	run(pool_base &pool, std::function<void()> tx, Locks &... locks)
 	{
-		if (pmemobj_tx_begin(pool.handle(), nullptr, TX_PARAM_NONE) !=
-		    0)
-			throw transaction_error("failed to start transaction");
+		int ret = 0;
+
+		if (pmemobj_tx_stage() == TX_STAGE_NONE) {
+			ret = pmemobj_tx_begin(pool.handle(), nullptr,
+					       TX_PARAM_CB,
+					       transaction::c_callback, nullptr,
+					       TX_PARAM_NONE);
+		} else {
+			ret = pmemobj_tx_begin(pool.handle(), nullptr,
+					       TX_PARAM_NONE);
+		}
+
+		if (ret != 0)
+			throw pmem::transaction_error(
+				"failed to start transaction")
+				.with_pmemobj_errormsg();
 
 		auto err = add_lock(locks...);
 
 		if (err) {
 			pmemobj_tx_abort(err);
 			(void)pmemobj_tx_end();
-			throw transaction_error(
-				"failed to add a lock to the transaction");
+			throw pmem::transaction_error(
+				"failed to add a lock to the transaction")
+				.with_pmemobj_errormsg();
 		}
 
 		try {
@@ -431,9 +440,9 @@ public:
 			pmemobj_tx_commit();
 		} else if (stage == TX_STAGE_ONABORT) {
 			(void)pmemobj_tx_end();
-			throw transaction_error("transaction aborted");
+			throw pmem::transaction_error("transaction aborted");
 		} else if (stage == TX_STAGE_NONE) {
-			throw transaction_error(
+			throw pmem::transaction_error(
 				"transaction ended prematurely");
 		}
 
@@ -475,12 +484,55 @@ public:
 	snapshot(const T *addr, size_t num = 1)
 	{
 		if (TX_STAGE_WORK != pmemobj_tx_stage())
-			throw transaction_error(
+			throw pmem::transaction_error(
 				"wrong stage for taking a snapshot.");
 
-		if (pmemobj_tx_add_range_direct(addr, sizeof(*addr) * num))
-			throw transaction_error(
-				"Could not take a snapshot of given memory range.");
+		if (pmemobj_tx_add_range_direct(addr, sizeof(*addr) * num)) {
+			if (errno == ENOMEM)
+				throw pmem::transaction_out_of_memory(
+					"Could not take a snapshot of given memory range.")
+					.with_pmemobj_errormsg();
+			else
+				throw pmem::transaction_error(
+					"Could not take a snapshot of given memory range.")
+					.with_pmemobj_errormsg();
+		}
+	}
+
+	/**
+	 * Possible stages of a transaction, for every stage one or more
+	 * callbacks can be registered.
+	 */
+	enum class stage {
+		work = TX_STAGE_WORK,	      /* transaction in progress */
+		oncommit = TX_STAGE_ONCOMMIT, /* successfully committed */
+		onabort = TX_STAGE_ONABORT,   /* tx_begin failed or transaction
+						 aborted */
+		finally = TX_STAGE_FINALLY,   /* ready for cleanup */
+	};
+
+	/**
+	 * Registers callback to be called on specified stage for the
+	 * transaction. In case of nested transactions those callbacks
+	 * are called when the outer most transaction enters a specified stage.
+	 *
+	 * @pre this function must be called during a transaction.
+	 *
+	 * @throw transaction_scope_error when called outside of a transaction
+	 * scope
+	 *
+	 * The typical usage example would be:
+	 * @snippet doc_snippets/transaction.cpp tx_callback_example
+	 */
+	static void
+	register_callback(stage stg, std::function<void()> cb)
+	{
+		if (pmemobj_tx_stage() != TX_STAGE_WORK)
+			throw pmem::transaction_scope_error(
+				"register_callback must be called during a transaction");
+
+		get_tx_data()->callbacks[static_cast<size_t>(stg)].push_back(
+			cb);
 	}
 
 private:
@@ -517,6 +569,66 @@ private:
 	{
 		return 0;
 	}
+
+	using callbacks_list_type = std::vector<std::function<void()>>;
+	using callbacks_map_type =
+		std::array<callbacks_list_type, MAX_TX_STAGE>;
+
+	/**
+	 * C-style function which is passed as callback to pmemobj_begin.
+	 * It executes previously registered callbacks for all stages.
+	 */
+	static void
+	c_callback(PMEMobjpool *pop, enum pobj_tx_stage obj_stage, void *arg)
+	{
+		/*
+		 * We cannot do anything when in TX_STAGE_NONE because
+		 * pmemobj_tx_get_user_data() can only be called when there is
+		 * an active transaction.
+		 */
+		if (obj_stage == TX_STAGE_NONE)
+			return;
+
+		auto *data = static_cast<tx_data *>(pmemobj_tx_get_user_data());
+		if (data == nullptr)
+			return;
+
+		for (auto &cb : data->callbacks[obj_stage])
+			cb();
+
+		/*
+		 * Callback for TX_STAGE_FINALLY is called as the last one so we
+		 * can free tx_data here
+		 */
+		if (obj_stage == TX_STAGE_FINALLY) {
+			delete data;
+			pmemobj_tx_set_user_data(NULL);
+		}
+	}
+
+	/**
+	 * This data is stored along with the pmemobj transaction data using
+	 * pmemobj_tx_set_data().
+	 */
+	struct tx_data {
+		callbacks_map_type callbacks;
+	};
+
+	/**
+	 * Gets tx user data from pmemobj or creates it if this is a first
+	 * call to this function inside a transaction.
+	 */
+	static tx_data *
+	get_tx_data()
+	{
+		auto *data = static_cast<tx_data *>(pmemobj_tx_get_user_data());
+		if (data == nullptr) {
+			data = new tx_data;
+			pmemobj_tx_set_user_data(data);
+		}
+
+		return data;
+	}
 };
 
 } /* namespace obj */
